data_model

Enum MazeCellState

pub enum MazeCellState {
    Empty,
    Wall,
    SolutionStep {
        value: usize,
    },
}
Represents the state of a maze cell

Variants

  • Empty: An empty cell that is not currently part of a solution path
  • Wall: A wall
  • SolutionStep - a cell that has been visited as the value’th step in an attempt to find the maze solution

Variants



Empty



Wall



SolutionStep

Fields

value: usize

Implementations

Source

impl MazeCellState

pub fn step_value(&self) -> Option<usize>

Returns the step value (if any) associated with the cell state instance

Returns

The step value (if variant is of type SolutionStep) else None