pub enum MazeCellState {
Empty,
Wall,
SolutionStep {
value: usize,
},
}Expand description
Represents the state of a maze cell
Variants
Empty: An empty cell that is not currently part of a solution pathWall: A wallSolutionStep- a cell that has been visited as thevalue’th step in an attempt to find the maze solution
Variants
Implementations
Sourceimpl MazeCellState
impl MazeCellState
Sourcepub fn step_value(&self) -> Option<usize>
pub fn step_value(&self) -> Option<usize>
Returns the step value (if any) associated with the cell state instance
Returns
The step value (if variant is of type SolutionStep) else None